<?php
/**
 * This class is used to add post meta boxes to all post types
 *
 * @author Kyle Benk <kjbenk@gmail.com>
 */

/**
 * Global Definitions
 */

/* Plugin Name */

if (!defined('WPSITE_LIMIT_TAGS_PLUGIN_NAME'))
    define('WPSITE_LIMIT_TAGS_PLUGIN_NAME', trim(dirname(plugin_basename(__FILE__)), '/'));

/* Plugin directory */

if (!defined('WPSITE_LIMIT_TAGS_PLUGIN_DIR'))
    define('WPSITE_LIMIT_TAGS_PLUGIN_DIR', WP_PLUGIN_DIR . '/' . WPSITE_LIMIT_TAGS_PLUGIN_NAME);

/* Plugin url */

if (!defined('WPSITE_LIMIT_TAGS_PLUGIN_URL'))
    define('WPSITE_LIMIT_TAGS_PLUGIN_URL', WP_PLUGIN_URL . '/' . WPSITE_LIMIT_TAGS_PLUGIN_NAME);

/* Plugin text-domain */

if (!defined('WPSITE_LIMIT_TAGS_PLUGIN_TEXT_DOMAIN'))
    define('WPSITE_LIMIT_TAGS_PLUGIN_TEXT_DOMAIN', 'wpsite-limit-tags');

add_action('add_meta_boxes', array('WPsiteAddMetaBox_LimitTags', 'wpsite_add_meta_box'), 10, 2);
add_action('save_post', array('WPsiteAddMetaBox_LimitTags', 'wpsite_save_meta_data'));
add_action('edit_attachment', array('WPsiteAddMetaBox_LimitTags', 'wpsite_save_meta_data'));

/**
 * Class used to add meta box
 *
 * @since 1.0.0
 */
class WPsiteAddMetaBox_LimitTags {

	private static $prefix = 'wpsite_limit_tags_meta_box_';

	private static $max_tags_post = 15;

	/**
	 * Load the text domain
	 *
	 * @since 1.0.0
	 */
	static function wpsite_load_textdoamin() {
		load_plugin_textdomain(WPSITE_LIMIT_TAGS_PLUGIN_TEXT_DOMAIN, false, WPSITE_LIMIT_TAGS_PLUGIN_DIR . '/languages');
	}

	/**
	 * Adds the meta box container.
	 */
	static function wpsite_add_meta_box($post_type, $post) {

		$settings = get_option('wpsite_limit_tags_settings');

		if ($settings === false)
			$settings = null;

		// Remove default tag meta_box if setting is true and user is not admin or editor.

		global $wp_roles;

		$current_user = wp_get_current_user();

		foreach ($wp_roles->role_names as $role_name => $role) {

			if (isset($settings['settings']['meta_box'][$role_name]) && $settings['settings']['meta_box'][$role_name] && in_array($role_name, $current_user->roles)) {
				break;
			} else if (isset($settings['settings']['meta_box'][$role_name]) && !$settings['settings']['meta_box'][$role_name] && in_array($role_name, $current_user->roles)) {
				remove_meta_box('tagsdiv-post_tag', $post_type, 'side');
			}
		}

		add_meta_box(
			self::$prefix,
			__('WPsite Tag Manager', WPSITE_LIMIT_TAGS_PLUGIN_TEXT_DOMAIN),
			array('WPsiteAddMetaBox_LimitTags' , 'wpsite_render_meta_box_content'),
			$post_type,
			'normal',
			'high'
		);
	}

	/**
	 * Render Meta Box content.
	 *
	 * @param WP_Post $post The post object.
	 */
	static function wpsite_render_meta_box_content($post) {

		// Add an nonce field so we can check for it later.
		wp_nonce_field(self::$prefix . 'inner_custom_box', self::$prefix . 'nonce' );

		$tags = get_post_meta( $post->ID, self::$prefix, true );

		$settings = get_option('wpsite_limit_tags_settings');

		if ($settings === false)
			$settings = null;

		if (isset($settings['tags'])) {

			?> <ul id="list_tags_add"> <?php
			foreach ($settings['tags'] as $tag => $val) {
				$val = isset($tags[$tag]) ? $tags[$tag] : false;
				?>
				<li class="wpsite_limit_tag_li <?php echo $val ? 'wpsite_limit_tag_li_active' : 'wpsite_limit_tag_li_inactive'; ?>" id="wpsite_limit_tag_li_<?php echo $tag; ?>">
					<input type="checkbox" id="<?php echo self::$prefix . $tag; ?>" name="<?php echo self::$prefix . $tag; ?>" <?php echo isset($val) && $val ? 'checked="checked"' :''; ?>/>
					<?php _e($tag, WPSITE_LIMIT_TAGS_PLUGIN_TEXT_DOMAIN); ?>
				</li>
				<?php
			}
			?> </ul> <?php
		}
	}

	/**
	 * Save the meta when the post is saved.
	 *
	 * @param int $post_id The ID of the post being saved.
	 */
	static function wpsite_save_meta_data($post_id) {

		/*
		* We need to verify this came from the our screen and with proper authorization,
		* because save_post can be triggered at other times.
		*/

		// Check if our nonce is set.
		if ( ! isset( $_POST[self::$prefix . 'nonce'] ) )
			return $post_id;

		$nonce = $_POST[self::$prefix . 'nonce'];

		// Verify that the nonce is valid.
		if ( ! wp_verify_nonce( $nonce, self::$prefix . 'inner_custom_box' ) )
		  return $post_id;

		// If this is an autosave, our form has not been submitted, so we don't want to do anything.
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
		  return $post_id;

		// Check the user's permissions.
		if ( 'page' == $_POST['post_type'] ) {

		if ( ! current_user_can( 'edit_page', $post_id ) )
		    return $post_id;

		} else {

		if ( ! current_user_can( 'edit_post', $post_id ) )
		    return $post_id;
		}

		/* OK, its safe for us to save the data now. */

		$settings = get_option('wpsite_limit_tags_settings');

		if ($settings === false)
			$settings = null;

		$all_tags = array();
		$add_tags = array();
		$remove_tags = array();

		if (isset($settings['tags'])) {

			foreach ($settings['tags'] as $tag => $val) {
				$all_tags[$tag] = isset($_POST[self::$prefix . $tag]) && $_POST[self::$prefix . $tag] ? true : false;

				if ($all_tags[$tag]) {
					if (count($add_tags) < (int) $settings['settings']['num_tags']) {
						$add_tags[] = $tag;
					} else {
						$all_tags[$tag] = false;
					}
				}
			}

			$terms = wp_get_post_tags($post_id);

			$temp_array = array();

			foreach ($terms as $term) {
				$temp_array[] = $term->name;
			}

			$i = 0;
			foreach ($temp_array as $term) {

				if (array_key_exists($term, $all_tags) && !$all_tags[$term]) {
					unset($temp_array[$i]);
				}
				$i++;
			}

			$terms = array_merge($temp_array, $add_tags);

			wp_set_post_tags($post_id, $terms, false);

			// Update the meta field in the database.
			update_post_meta($post_id, self::$prefix, $all_tags);
		}
	}
}